#!/usr/bin/env python
# coding=UTF-8
from __future__ import (unicode_literals, division, absolute_import,
                        print_function)

__license__   = 'GPL v3'
__copyright__ = '2012, Tomáš Pěnička <penicka@gmail.com>'
__docformat__ = 'restructuredtext en'

import os
from functools import partial
import shutil
import filecmp
from PyQt4.Qt import QMenu, QToolButton, QPixmap, Qt
from calibre.gui2 import info_dialog
from calibre.gui2.actions import InterfaceAction

import calibre_plugins.correct_encoding.config as cfg
from calibre_plugins.correct_encoding.main import EncodingFixer

from calibre_plugins.correct_encoding.common_utils import (set_plugin_icon_resources, get_icon,
                                                     create_menu_action_unique)
from calibre.ptempfile import TemporaryFile

PLUGIN_ICONS = ['images/correct_encoding.png']

class CorrectEncodingAction(InterfaceAction):

    name = 'Correct Encoding'
    # Create our top-level menu/toolbar action (text, icon_path, tooltip, keyboard shortcut)
    action_spec = ('Correct Encoding', None, None, None)
    popup_type = QToolButton.MenuButtonPopup
    action_type = 'current'

    def genesis(self):
        self.menu = QMenu(self.gui)
        self.old_actions_unique_map = {}
        self.default_translation_data = None

        # Read the plugin icons and store for potential sharing with the config widget
        icon_resources = self.load_resources(PLUGIN_ICONS)
        set_plugin_icon_resources(self.name, icon_resources)


        self.rebuild_menus()

        # Assign our menu to this action and an icon
        self.qaction.setMenu(self.menu)
        self.qaction.setIcon(get_icon(PLUGIN_ICONS[0]))
        self.qaction.triggered.connect(self.correct_encodings)
        self.menu.aboutToShow.connect(self.about_to_show_menu)

    def about_to_show_menu(self):
        self.rebuild_menus()

    def rebuild_menus(self):
        c = cfg.plugin_prefs[cfg.STORE_NAME]
        translation_data_items = c[cfg.KEY_PROFILES]
        m = self.menu
        m.clear()
        self.actions_unique_map = {}

        for translation_data in translation_data_items:
            menu_text = 'Profile: {}'.format(translation_data[cfg.KEY_PROFILE])
            is_default = bool(translation_data[cfg.KEY_DEFAULT])
            ac = create_menu_action_unique(self, m, menu_text, is_checked=is_default,
                                           triggered=partial(self.correct_encodings, translation_data[cfg.KEY_TRANSLATION] ))
            self.actions_unique_map[ac.calibre_shortcut_unique_name] = ac.calibre_shortcut_unique_name

            if is_default:
                self.default_translation_data = translation_data
        m.addSeparator()
        create_menu_action_unique(self, m, _('&Customize plugin')+'...', 'config.png',
                                  shortcut=False, triggered=self.show_configuration)

        # Before we finalize, make sure we delete any actions for menus that are no longer displayed
        for menu_id, unique_name in self.old_actions_unique_map.iteritems():
            if menu_id not in self.actions_unique_map:
                self.gui.keyboard.unregister_shortcut(unique_name)
        self.old_actions_unique_map = self.actions_unique_map
        self.gui.keyboard.finalize()

    def correct_encodings(self, translation=None):
        if not translation:
            translation = self.default_translation_data[cfg.KEY_TRANSLATION]

        rows = self.gui.library_view.selectionModel().selectedRows()
        if not rows or len(rows) == 0:
            return
        current_idx = self.gui.library_view.currentIndex()
        db = self.gui.library_view.model().db
        ids = set(self.gui.library_view.get_selected_ids())
        corrected_count = 0
        equal_count = 0
        for book_id in ids:
            if db.has_format(book_id, "txt", index_is_id=True):
                path_to_text = db.format_abspath(book_id, "txt", index_is_id=True)
                res =  self.correct_encoding_for_book(path_to_text, translation)
                corrected_count += 1
                if res:
                    equal_count += 1

        if corrected_count == 0:
            return info_dialog(self.gui, 'No txt files converted', 'None of the selected book(s) have  format .txt',
                            show=True, show_copy_button=False)
        else:
            return info_dialog(self.gui, 'Files corrected', '{} files processed, {} were equal'.format(corrected_count, equal_count),
                            show=True, show_copy_button=False)

    def correct_encoding_for_book(self, path_to_book, translation):
        result = True
        table = {}
        for line in translation.splitlines():
            line = line.strip()
            if line:
                if '#' == line[0]:
                    continue  # comment
                vals = line.split(None, 1)
                key = vals[0]
                if len(vals) == 1:
                    replacement = " " # if I found nothing on the line, there was probably a space
                else:
                    replacement = vals[1].strip()
                table[int(key, 16)] = replacement
        fixer = EncodingFixer()
        fh = open(path_to_book, "rb")
        with TemporaryFile(suffix = "txt", mode = "wb") as filename:
            fo = open(filename, "wb")
            fixer.run(fh, fo, table)
            fh.close()
            fo.flush()
            fo.close()
            result = filecmp.cmp(filename, path_to_book, shallow=False)  # False if different
            shutil.move(fo.name, path_to_book)
        return result  # False if different

    def show_configuration(self):
        self.interface_action_base_plugin.do_user_config(self.gui)
